/* NOR shadow-page test
 * Copyright (C) 2013 Daniel Beer <dlbeer@gmail.com>
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include "shadow.h"

static nor_word_t test_block[NOR_WORDS * 2];
static shadow_state_t test_state;

static int seq_block(int seed, uint8_t *blk)
{
	int i;

	srandom(seed);
	for (i = 0; i < SHADOW_PAYLOAD_SIZE; i++)
		blk[i] = random();

	return random() % SHADOW_PAYLOAD_SIZE;
}

static void test_seq(int seed)
{
	uint8_t ref[SHADOW_PAYLOAD_SIZE];
	uint8_t in[SHADOW_PAYLOAD_SIZE];
	struct shadow_reader r;
	int len = seq_block(seed, ref);

	shadow_read_begin(&r, test_state, test_block);
	while (shadow_read_ptr(&r) < len) {
		const int q = random() % (len - shadow_read_ptr(&r) + 1);
		const int a = shadow_read(&r, in + shadow_read_ptr(&r), q);

		assert(q == a);
	}
	shadow_read_end(&r);

	assert(!memcmp(ref, in, len));
}

static shadow_state_t write_seq(int seed)
{
	uint8_t ref[SHADOW_PAYLOAD_SIZE];
	struct shadow_writer w;
	int len = seq_block(seed, ref);

	shadow_write_begin(&w, test_state, test_block);
	while (shadow_write_ptr(&w) < len) {
		const int q = random() % (len - shadow_write_ptr(&w) + 1);
		const int a = shadow_write(&w, ref + shadow_write_ptr(&w), q);

		assert(q == a);
	}

	return shadow_write_end(&w, test_state);
}

static void corrupt_last(void)
{
	nor_block_t which =
		(test_state & SHADOW_FLAG_ALT_BLOCK) ?
		nor_shadow(test_block) : test_block;

	which[random() % NOR_WORDS] ^= 1;
}

int main(void)
{
	int i;

	printf("init\n");
	test_state = shadow_scan(test_block);
	assert(!shadow_has_data(test_state));

	printf("write\n");
	for (i = 0; i < 100; i++) {
		const shadow_state_t next = write_seq(i);

		assert((next ^ test_state) & SHADOW_FLAG_ALT_BLOCK);
		assert(shadow_has_data(next));
		test_state = next;

		test_seq(i);
		assert(shadow_scan(test_block) == test_state);
	}

	printf("rollback\n");
	for (i = 0; i < 100; i++) {
		write_seq(i);
		test_state = shadow_scan(test_block);
		test_seq(i);

		write_seq(i + 1);
		test_state = shadow_scan(test_block);
		test_seq(i + 1);

		write_seq(i + 2);
		test_state = shadow_scan(test_block);
		test_seq(i + 2);

		corrupt_last();
		test_state = shadow_scan(test_block);
		test_seq(i + 1);
	}

	return 0;
}
